<?php
namespace Teknisa\FAI\Modules\Relatorio\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\Report\ReportService;

class Fai40000EstatisticaFaturamentoReport extends \Zeedhi\Framework\Controller\Report {
    
    protected $birtStrategy;
    protected $connection;
    protected $environment;
    protected $newCode;
    protected $crudService;
    protected $entityManager;
    public function __construct(ReportService $reportService, $birtStrategy, $connection,$environment,$newCode,$crudService,$entityManager) {
        $this->birtStrategy  = $birtStrategy;
        $this->reportService  = $reportService;
        $this->connection = $connection;
        $this->environment   = $environment;
        $this->newCode   = $newCode;
        $this->crudService  = $crudService;
        $this->entityManager       = $entityManager;
    }


    protected $reportMapping = array (
        'FAI40000' =>
        array (
            'strategy'         => 'BIRT',
            'parameterMapping' =>
                array (
                    'LCDFILIAL'  => 'L_CDFILIAL',
                    'LCDFILIFISC'  => 'L_CDFILIFISC',
                    'LCHAVE'  => 'L_CDSERIE',
                    'LCDPAIS'  => 'L_CDPAIS',
                    'LSGESTADO'  => 'L_SGESTADO',
                    'LCDTIPOFATUR'  => 'L_CDTIPOFATUR',
                    'P_DTINICIALANT' => 'P_DTINICIALANT',
                    'P_DTFINALANT'   => 'P_DTFINALANT',
                    'P_DTINICIAL'    => 'P_DTINICIAL',
                    'P_DTFINAL'      => 'P_DTFINAL',
                    'P_CDTIPOOPER'   => 'P_CDTIPOOPER',
                    'P_TCDTIPOOPER'  => 'P_TCDTIPOOPER',
                    'P_OPORDENACAO'  => 'P_OPORDENACAO',
                    'VALORINICIAL'  => 'P_VALORINI',
                    'VALORFINAL'    => 'P_VALORFIN',
                    'NRORG'         => 'P_NRORG'
                ),
            ),
    );

    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        //ini_set('default_socket_timeout', 40000);
        $row = $request->getRow();
        /*
        substitui o ponto(.) dos campos VALORINICICAL e VALORFINAL por vazio('')
        devido conflitos no banco de dados que contem formato PT-BR, gerando erro     
        */
        $row['VALORINICIAL'] = str_replace('.','',$row['VALORINICIAL']);
        $row['VALORFINAL'] = str_replace('.','',$row['VALORFINAL']);
        $row['NRORG']=$this->environment->getNrOrg();
        if($row['P_CDTIPOOPER']==='T'){
            $row['P_CDTIPOOPER']='';
        }

       if($row['LCDFILIAL']==='T'){
           $row['LCDFILIAL']=$this->getAllFiliApurac($row['LCDFILIFISC'],$row['NRORG']);
       }
      
       if($row['LCDFILIFISC']==='T'){
           $row['LCDFILIFISC']=$this->getAllFiliFisc();
       }

        //retorna estados pelos paises
        if($row['LSGESTADO']==='T'){
            $row['LSGESTADO']=$this->getAllEstados($row['LCDPAIS'], $row['NRORG']);
        }
        //retorna paises
        if($row['LCDPAIS']==='T'){
            $row['LCDPAIS']=$this->getAllPais();
        }
        if($row['LCHAVE']==='T'){
            $row['LCHAVE']=$this->getAllCHAVE();
        }
        if($row['LCDTIPOFATUR']==='T'){
            $row['LCDTIPOFATUR']=$this->getAllTipoFatur($row['NRORG']);
        }
        
        
        $row['LCDPAIS']=$this->criarListaValores($row['LCDPAIS']);
       
        $row['LSGESTADO']=$this->criarListaValores($row['LSGESTADO']);
        $row['LCHAVE']=$this->criarListaValores($row['LCHAVE']);

        $row['LCDTIPOFATUR']=$this->criarListaValores($row['LCDTIPOFATUR']);
         
        $row['LCDFILIAL']=$this->criarListaValores($row['LCDFILIAL']);
       
        $row['LCDFILIFISC']=$this->criarListaValores($row['LCDFILIFISC']);



        $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
    }

//   //  private function setProperty($row, $arrayMap){
//         foreach(array_keys($arrayMap) as $key){
//             foreach(array_values($this->reportMapping['FAI40000']['parameterMapping']) as $keyReport){
//                 $normalizedName = str_replace('_', '', $keyReport);
//                 if(strpos($normalizedName, $key) !== false){
//                     $row[$keyReport] = $arrayMap[$key];
//                 }
//             }
//         }
//     }
    
    private function getAllFiliFisc(){
       $conditions = array();
        $rows = $this->crudService->getFilteredDataSet('filial_filial', $conditions)->getRows();
        
        $filifisc= array();
        
        foreach($rows as $row) {
            array_push($filifisc, $row['CDFILIFISC']);
        }
        $filifisc=implode("_",$filifisc);
        return $filifisc;
    }
    
    private function getAllFiliApurac($filiFisc,$nrorg){
        $conditions = array(
            array(
                "columnName" => "CDFILIFISC",
                "operator" => "IN",
                "value" => $filiFisc
            ),
             array(
                "columnName" => "NRORG",
                "operator" => "IN",
                "value" => $nrorg
            )
        );
        $rows = $this->crudService->getFilteredDataSet('fai40000_filial_apuracao', $conditions)->getRows();
        
        $filiais= array();
        
        foreach($rows as $row) {
            array_push($filiais, $row['CDFILIAL']);
        }
        $filiais=implode("_",$filiais);
        return $filiais;
    
    }
    
    public function getAllPais() {
        $conditions = array();
        $rows = $this->crudService->getFilteredDataSet('pais', $conditions)->getRows();
        
        $paises= array();
        
        foreach($rows as $row) {
            array_push($paises, $row['CDPAIS']);
        }
        $paises=implode("_",$paises);
        return $paises;
    }
    
    public function getAllEstados($estados,$nrorg){
        $conditions = array(
            array(
                "columnName" => "CDPAIS",
                "operator" => "IN",
                "value" => $estados
            ),
             array(
                "columnName" => "NRORG",
                "operator" => "IN",
                "value" => $nrorg
            ),
        );
        $rows = $this->crudService->getFilteredDataSet('fai40000_estado_por_pais', $conditions)->getRows();
        
        $estados= array();
        
        foreach($rows as $row) {
            array_push($estados, $row['CDPAIS'].$row['SGESTADO']);
        }
        $estados=implode("_",$estados);
        return $estados;
    }
    
    public function getAllCHAVE(){

         $sql = "SELECT CDEMPRESA || CDINSCESTA || CDINSCMUNI || CDSERIE AS CHAVE, CDSERIE, DSSERIE, CDEMPRESA, CDINSCESTA, CDINSCMUNI, NRORG FROM SERIENF ORDER BY CDSERIE";

        try {
            $chaves = $this->entityManager->getConnection()->fetchAll($sql);
            $series= array();
            foreach($chaves as $row) {
                
                array_push($series, $row['CHAVE']);
            }
            $series=implode("_",$series);
            return $series;
            
          
        } catch (\Exception $e) {
            throw $e;
        }

       
    }
    
    public function getAllTipoFatur($nrorg){
        $conditions =  $conditions = array(
             array(
                "columnName" => "NRORG",
                "operator" => "IN",
                "value" => $nrorg
            ),
        );
        $rows = $this->crudService->getFilteredDataSet('fai40000_tipo_contrato', $conditions)->getRows();
        
        $tipofatur= array();
        
        foreach($rows as $row) {
            array_push($tipofatur, $row['CDTIPOFATUR']);
        }
        $tipofatur=implode("_",$tipofatur);
        return $tipofatur;
    }

    public function criarListaValores($valores){
       
            $this->entityManager->beginTransaction();
            $lista = explode('_',$valores);
            $nrlista = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrorgTrab(), 1, 10, false);
            try{
                foreach($lista as $cdlista){
                    $this->insertLista($nrlista, $cdlista);
                }
                $this->entityManager->commit();
                return $nrlista;
            } catch(\Exception $e){
                $this->entityManager->rollback();
                throw $e;
            }
        
    }
    public function insertLista($nrlista, $cdlista) {
        $params = array(
            "NRLISTA" => $nrlista,
            "CDLISTA" => $cdlista,
            "NRORG" => $this->environment->getNrorgTrab()
        );
        
        if(strlen($cdlista)>0){
            $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA, NRORG) VALUES (:NRLISTA, :CDLISTA, :NRORG)";
            $query = $this->connection->prepare($sql);
            $query->execute($params);
        }
    }
}